﻿DeclareModule Share
UseModule AIMP

Declare Init()
Declare Free(AIMP=#False)
Declare SetCB(*CB, Flag.a)
Declare IsData()
Declare GetShare(*Data.TAIMPVisualData)
Declare SetShare(*Data.TAIMPVisualData)
EndDeclareModule

Module Share

EnableExplicit

#Mutex_TimeOut = 4000
#Event_TimeOut = 400

Prototype pCB(*Buff)

Structure ShareData
  VisualData.TAIMPVisualData
  CountProcess.l ; Количество процессов, в которые загружена DLL.
  Counter.l      ; При при наличии новых данных записывается число из CountProcess. 
                 ; Когда процесс получит данные, необходимо уменишить значение на 1.
EndStructure

Structure Params
  hMutex.i
  hEvent.i
  ThreadID.i
  BreakThread.a
  *CB.pCB
  SendDataCB.a
  FlagNewData.a
EndStructure

Global *gShareData.ShareData, gParam.Params

Procedure Thread(*x)
  Protected VisualData.TAIMPVisualData
  
  With gParam
    
    Repeat
      Delay(10)
      
      If WaitForSingleObject_(\hEvent, #Event_TimeOut) = #WAIT_OBJECT_0
        
        If \BreakThread = #True
          Break
        EndIf
        
        If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
          
          \FlagNewData = #True
        
          *gShareData\Counter - 1
          If *gShareData\Counter <= 0
            ResetEvent_(\hEvent)
          EndIf
          
          If \CB And \SendDataCB = #True
            CopyStructure(@*gShareData\VisualData, @VisualData, TAIMPVisualData)
          EndIf
          
          ReleaseMutex_(\hMutex)
        EndIf
        
        If \CB
          If \SendDataCB = #True
            \CB(@VisualData)
          Else
            \CB(0)
          EndIf
        EndIf
        
      EndIf
      
    Until \BreakThread = #True
    \BreakThread+1
  EndWith
EndProcedure

Procedure SetCB(*CB, Flag.a)
  With gParam
    \CB = *CB
    \SendDataCB = Flag
  EndWith
EndProcedure

Procedure IsData()
  ProcedureReturn gParam\FlagNewData
EndProcedure

Procedure GetShare(*Data.TAIMPVisualData)
  Protected Res = #False
  
  With gParam
    If \hMutex And \hEvent And *Data
      If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
        CopyMemory(@*gShareData\VisualData, *Data, SizeOf(TAIMPVisualData))
        \FlagNewData = #False
        ReleaseMutex_(\hMutex)
        Res = #True
      EndIf
    EndIf
  EndWith
  
  ProcedureReturn Res
EndProcedure


Procedure SetShare(*Data.TAIMPVisualData)
  Protected Event = #False
  
  With gParam
    If \hMutex And \hEvent And *Data
      
      If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
        If *gShareData\CountProcess > 0 ; Если DLL загружена в другие процессы кроме плеера.
          *gShareData\Counter = *gShareData\CountProcess
          CopyMemory(*Data, @*gShareData\VisualData, SizeOf(TAIMPVisualData))
          Event = #True
        EndIf
        ReleaseMutex_(\hMutex)
      EndIf
      
      If Event = #True
        SetEvent_(\hEvent)
      EndIf
      
    EndIf
  EndWith
EndProcedure


Procedure Init()
  Protected Str.s
  
  *gShareData = ?ShareData
  
  UseMD5Fingerprint()
  Str = StringFingerprint(ProgramFilename(), #PB_Cipher_MD5)
  
  With gParam
    \CB = 0
    \hMutex = CreateMutex_(0, 0, "m"+Str)
    If \hMutex
      
      \hEvent = CreateEvent_(0, #True, #False, "e"+Str)
      
      \BreakThread = #False
      \ThreadID = CreateThread(@Thread(), 0)
      
      If \ThreadID
        If WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
          *gShareData\CountProcess + 1
          ReleaseMutex_(\hMutex)
        EndIf
      EndIf
      
    EndIf
  EndWith
EndProcedure

Procedure Free(AIMP=#False)
  Protected i
  
  With gParam
    
    If \ThreadID
      
      If IsThread(\ThreadID)
        
        \BreakThread = #True
        If AIMP = #True And \hEvent ; DLL загружена в процесс плеера.
          SetEvent_(\hEvent)        ; Чтобы быстрее завершился поток.
        EndIf
        
        For i=0 To #Event_TimeOut
          If \BreakThread = #True+1
            Break
          Else
            Delay(2)
          EndIf
        Next i
        
        If \BreakThread <> #True+1 And IsThread(\ThreadID)
          KillThread(\ThreadID)
        EndIf
      EndIf
      
      If \hMutex And WaitForSingleObject_(\hMutex, #Mutex_TimeOut) = #WAIT_OBJECT_0
        *gShareData\CountProcess - 1
        ReleaseMutex_(\hMutex)
      EndIf
      
      \ThreadID = 0
    EndIf
    
    If AIMP <> #True
      
      If \hMutex
        CloseHandle_(\hMutex)
        \hMutex = 0
      EndIf
      
      If \hEvent
        CloseHandle_(\hEvent)
        \hEvent=0
      EndIf
      
    EndIf
    
  EndWith
EndProcedure
  
DataSection
  ! section 'data' data readable writeable shareable
  ShareData:
  ! db 8192 dup (0)
  ! section '.data' data readable writeable
EndDataSection

EndModule
; IDE Options = PureBasic 5.42 LTS (Windows - x86)
; Folding = --
; EnableUnicode
; EnableXP